/* slurp-example-simple.c
 *
 * Copyright (C) 2017 Julian Richen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <slurp/slurp.h>
#include <glib.h>

static void
progress_callback (GObject  *object,
                   gint64    downloaded_bytes,
                   gint64    total_bytes,
                   gint64    download_speed,
                   gint64    estimated_time,
                   gpointer  user_data)
{
    g_print ("[ %06.2f%% ] %s / %s (%s/s) Done in %lis\n",
             (gfloat) (((gfloat) downloaded_bytes / (gfloat) total_bytes) * 100),
             g_format_size (total_bytes),
             g_format_size (downloaded_bytes),
             g_format_size (download_speed),
             estimated_time);
}

static void
download_callback (GObject      *object,
                   GAsyncResult *result,
                   gpointer      user_data)
{
    g_autoptr (GMainLoop) loop = (GMainLoop *) user_data;
    g_autoptr (GError) error = NULL;
    gboolean retval = FALSE;

    g_return_if_fail (SLURP_IS_DOWNLOAD (object));
    g_return_if_fail (G_IS_ASYNC_RESULT (result));
    g_return_if_fail (g_task_is_valid (result, object));

    retval = slurp_download_resource_finish (object, result, &error);

    if (!retval)
        g_print ("Failed to downloaded resource: %s\n", error->message);
    else
        g_print ("Successfully downloaded resource\n");

    g_main_loop_quit (loop);
}

int
main (int    argc,
      char **argv)
{
/*
    gchar *uri = "file:///home/julian/Downloads/libslurp/tests/resource.png";
    //gchar *uri = "file:///home/julian/Downloads/libslurp/tests/r.esou.rce.tar.gz";

    g_print ("FILE_NAME: %s\n", slurp_utils_uri_get_file_name (uri));
    g_print ("EXTENSION: %s\n", slurp_utils_uri_get_file_extention (uri));
    g_print ("VALID URI: %s\n", slurp_utils_uri_create_valid_target (uri, FALSE));

    return 0;
*/

    g_autoptr (GMainLoop) loop = NULL;
    g_autoptr (SlurpDownload) download = NULL;
    g_autoptr (SlurpResource) resource = NULL;
    g_autoptr (GCancellable) cancellable = NULL;

    loop = g_main_loop_new (NULL, FALSE);

    download = g_object_new (SLURP_TYPE_DOWNLOAD, NULL);

    resource = g_object_new (SLURP_TYPE_RESOURCE,
                             //"source", "https://download.fedoraproject.org/pub/fedora/linux/releases/27/Workstation/x86_64/iso/Fedora-Workstation-Live-x86_64-27-1.6.iso",
                             //"target", "file:///home/julian/Downloads/libslurp/tests/fedora.iso",
                             "source", "https://www.google.com/images/branding/googlelogo/1x/googlelogo_color_272x92dp.png",
                             "target", "file:///home/julian/Downloads/libslurp/tests/resource.png",
                             "overwrite", TRUE,
                             NULL);

    g_signal_connect (resource,
                      "progress",
                      G_CALLBACK (progress_callback),
                      NULL);

    cancellable = g_cancellable_new ();

    slurp_download_resource_async (download,
                                   resource,
                                   cancellable,
                                   download_callback,
                                   g_main_loop_ref (loop));

    g_main_loop_run (loop);

    return 0;
}
