/* slurp-download-data.c
 *
 * Copyright © 2018 Julian Richen <julian@richen.io>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "slurp-download-data.h"

G_DEFINE_BOXED_TYPE (SlurpDownloadData, slurp_download_data, slurp_download_data_ref, slurp_download_data_unref)

SlurpDownloadData *
slurp_download_data_new (void)
{
    SlurpDownloadData *self;

    self = g_slice_new0 (SlurpDownloadData);
    self->ref_count = 1;

    return self;
}

SlurpDownloadData *
slurp_download_data_copy (SlurpDownloadData *self)
{
    SlurpDownloadData *copy;

    g_return_val_if_fail (self, NULL);
    g_return_val_if_fail (self->ref_count, NULL);

    copy = slurp_download_data_new ();

    return copy;
}

static void
slurp_download_data_free (SlurpDownloadData *self)
{
    g_assert (self);
    g_assert_cmpint (self->ref_count, ==, 0);

    g_clear_pointer (&self->resource, g_object_unref);

    if (g_main_loop_is_running (self->loop))
        g_main_loop_quit (self->loop);

    g_clear_pointer (&self->loop, g_main_loop_unref);
    g_clear_pointer (&self->input, g_object_unref);
    g_clear_pointer (&self->file, g_object_unref);
    g_clear_pointer (&self->output, g_object_unref);

    g_slice_free (SlurpDownloadData, self);
}

SlurpDownloadData *
slurp_download_data_ref (SlurpDownloadData *self)
{
    g_return_val_if_fail (self, NULL);
    g_return_val_if_fail (self->ref_count, NULL);

    g_atomic_int_inc (&self->ref_count);

    return self;
}

void
slurp_download_data_unref (SlurpDownloadData *self)
{
    g_return_if_fail (self);
    g_return_if_fail (self->ref_count);

    if (g_atomic_int_dec_and_test (&self->ref_count))
        slurp_download_data_free (self);
}
